@@ -6,6 +6,8 @@
 
 #include "settings/Settings.h"
 #include "utils/AABB3D.h"
+#include "utils/floatpoint.h"
+#include "utils/FMatrix4x3.h"
 
 namespace cura
 {
@@ -92,6 +94,12 @@ public:
         aabb.offset(offset);
     }
 
+    /*!
+     * Apply an affine transformation to this mesh's 3D data.
+     * \param transformation The transformation to apply.
+     */
+    void transform(const FMatrix4x3& transformation);
+
 private:
     mutable bool has_disconnected_faces; //!< Whether it has been logged that this mesh contains disconnected faces
     mutable bool has_overlapping_faces; //!< Whether it has been logged that this mesh contains overlapping faces
