@@ -1,4 +1,4 @@
-//Copyright (c) 2018 Ultimaker B.V.
+//Copyright (c) 2021 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
 #include <map>
@@ -147,22 +147,46 @@ int PathOrderOptimizer::getClosestPointInPolygon(Point prev_point, int poly_idx)
 {
     ConstPolygonRef poly = *polygons[poly_idx];
 
-    int best_point_idx = -1;
-    float best_point_score = std::numeric_limits<float>::infinity();
-    Point p0 = poly.back();
+    // Find most extreme point in one direction*. For the 'actual loop' (see below), start from this point,
+    // so it can act as a 'tie breaker' if all differences in dist-score for a polygon fall within epsilon.
+    // *) Direction/point should be equal to user-specified point if available, should be an arbitrary point outside of the BP otherwise.
+    constexpr coord_t EPSILON = 25; // = 5^2 square micron
+    unsigned int start_from_pos = 0;
+    const Point focus_fixed_point =
+        (config.type == EZSeamType::USER_SPECIFIED) ?
+        config.pos :
+        Point(0, std::sqrt(std::numeric_limits<coord_t>::max()));  // NOTE: Use sqrt, so the squared size can be used when comparing distances.
+    coord_t smallest_dist_sqd = std::numeric_limits<coord_t>::max();
     for (unsigned int point_idx = 0; point_idx < poly.size(); point_idx++)
     {
+        const coord_t dist_sqd = vSize2(focus_fixed_point - poly[point_idx]);
+        if (dist_sqd < smallest_dist_sqd)
+        {
+            start_from_pos = point_idx;
+            smallest_dist_sqd = dist_sqd;
+        }
+    }
+    const unsigned int end_before_pos = poly.size() + start_from_pos;
+
+    // Loop over the polygon to find the 'best' index given all the parameters.
+    int best_point_idx = -1;
+    float best_point_score = std::numeric_limits<float>::infinity();
+    Point p0 = poly[(start_from_pos - 1 + poly.size()) % poly.size()];
+    for (unsigned int point_idx_without_modulo = start_from_pos; point_idx_without_modulo < end_before_pos; point_idx_without_modulo++)
+    {
+        const unsigned int point_idx = point_idx_without_modulo % poly.size();
+
         const Point& p1 = poly[point_idx];
         const Point& p2 = poly[(point_idx + 1) % poly.size()];
         // when type is SHARPEST_CORNER, actual distance is ignored, we use a fixed distance and decision is based on curvature only
-        float dist_score = (config.type == EZSeamType::SHARPEST_CORNER && config.corner_pref != EZSeamCornerPrefType::Z_SEAM_CORNER_PREF_NONE)? 10000 : vSize2(p1 - prev_point);
+        float dist_score = (config.type == EZSeamType::SHARPEST_CORNER && config.corner_pref != EZSeamCornerPrefType::Z_SEAM_CORNER_PREF_NONE) ? MM2INT(10) : vSize2(p1 - prev_point);
         const float corner_angle = LinearAlg2D::getAngleLeft(p0, p1, p2) / M_PI; // 0 -> 2
         float corner_shift;
         if (config.type == EZSeamType::SHORTEST)
         {
             // the more a corner satisfies our criteria, the closer it appears to be
             // shift 10mm for a very acute corner
-            corner_shift = 10000 * 10000;
+            corner_shift = MM2INT(10) * MM2INT(10);
         }
         else
         {
@@ -175,7 +199,7 @@ int PathOrderOptimizer::getClosestPointInPolygon(Point prev_point, int poly_idx)
         switch (config.corner_pref)
         {
             case EZSeamCornerPrefType::Z_SEAM_CORNER_PREF_INNER:
-                if (corner_angle > 1)
+                if (corner_angle > 1) //Is an inner corner.
                 {
                     // p1 lies on a concave curve so reduce the distance to favour it
                     // the more concave the curve, the more we reduce the distance
@@ -183,7 +207,7 @@ int PathOrderOptimizer::getClosestPointInPolygon(Point prev_point, int poly_idx)
                 }
                 break;
             case EZSeamCornerPrefType::Z_SEAM_CORNER_PREF_OUTER:
-                if (corner_angle < 1)
+                if (corner_angle < 1) //Is an outer corner.
                 {
                     // p1 lies on a convex curve so reduce the distance to favour it
                     // the more convex the curve, the more we reduce the distance
@@ -198,7 +222,7 @@ int PathOrderOptimizer::getClosestPointInPolygon(Point prev_point, int poly_idx)
             {
                 //More curve is better score (reduced distance), but slightly in favour of concave curves.
                 float dist_score_corner = fabs(corner_angle - 1) * corner_shift;
-                if (corner_angle < 1)
+                if (corner_angle > 1) //Is an inner corner.
                 {
                     dist_score_corner *= 2;
                 }
@@ -210,7 +234,7 @@ int PathOrderOptimizer::getClosestPointInPolygon(Point prev_point, int poly_idx)
                 // do nothing
                 break;
         }
-        if (dist_score < best_point_score)
+        if ((dist_score - EPSILON) < best_point_score)
         {
             best_point_idx = point_idx;
             best_point_score = dist_score;
@@ -465,7 +489,7 @@ float LineOrderOptimizer::combingDistance2(const Point &p0, const Point &p1)
     if (loc_to_line == nullptr)
     {
         // do the initialisation required to be able to calculate realistic travel distances to the start of new paths
-        loc_to_line = PolygonUtils::createLocToLineGrid(*combing_boundary, 1000); // 1mm grid to reduce computation time
+        loc_to_line = PolygonUtils::createLocToLineGrid(*combing_boundary, MM2INT(1)); // 1mm grid to reduce computation time
     }
 
     CombPath comb_path;
@@ -483,7 +507,7 @@ float LineOrderOptimizer::combingDistance2(const Point &p0, const Point &p1)
 
     // couldn't comb, fall back to a large distance
 
-    return vSize2f(p1 - p0) * 10000;
+    return vSize2f(p1 - p0) * MM2INT(10);
 }
 
 /*
