@@ -89,7 +89,7 @@ struct LayerIndex
         operator++(); //Increment myself.
         return original_value;
     }
-    LayerIndex operator --()
+    LayerIndex& operator --()
     {
         value--;
         return *this;
@@ -123,4 +123,4 @@ namespace std
     };
 }
 
-#endif //LAYERINDEX_H
\ No newline at end of file
+#endif //LAYERINDEX_H
