@@ -17,4 +17,10 @@ bool ProximityPointLink::operator==(const ProximityPointLink& other) const
     return (a == other.a && b == other.b) || (a == other.b && b == other.a);
 }
 
+void ProximityPointLink::setDist(coord_t distance) const
+{
+    ProximityPointLink& thiss = *const_cast<ProximityPointLink*>(this);
+    thiss.dist = distance;
+}
+
 }//namespace cura 
