@@ -1,4 +1,4 @@
-//Copyright (c) 2018 Ultimaker B.V.
+//Copyright (c) 2021 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
 #include <list>
@@ -9,6 +9,7 @@
 #include "polygonUtils.h"
 #include "SparsePointGridInclusive.h"
 #include "../utils/logoutput.h"
+#include "../infill.h"
 
 #ifdef DEBUG
 #include "AABB.h"
@@ -83,6 +84,84 @@ void PolygonUtils::spreadDots(PolygonsPointIndex start, PolygonsPointIndex end,
     assert(result.size() == n_dots && "we didn't generate as many wipe locations as we asked for.");
 }
 
+std::vector<Point> PolygonUtils::spreadDotsArea(const Polygons& polygons, coord_t grid_size)
+{
+    Infill infill_gen(EFillMethod::LINES, false, false, polygons, 0, 0, grid_size, 0, 1, 0, 0, 0, 0, 0);
+    Polygons result_polygons;
+    Polygons result_lines;
+    infill_gen.generate(result_polygons, result_lines);
+    std::vector<Point> result;
+    for (PolygonRef line : result_lines)
+    {
+        assert(line.size() == 2);
+        Point a = line[0];
+        Point b = line[1];
+        assert(a.X == b.X);
+        if (a.Y > b.Y)
+        {
+            std::swap(a, b);
+        }
+        for (coord_t y = a.Y - (a.Y % grid_size) - grid_size; y < b.Y; y += grid_size)
+        {
+            if (y < a.Y) continue;
+            result.emplace_back(a.X, y);
+        }
+    }
+    return result;
+}
+
+bool PolygonUtils::lineSegmentPolygonsIntersection(const Point& a, const Point& b, const Polygons& current_outlines, const LocToLineGrid& outline_locator, Point& result, const coord_t within_max_dist)
+{
+    const coord_t within_max_dist2 = within_max_dist * within_max_dist;
+
+    Point coll;
+    coord_t closest_dist2 = within_max_dist2;
+
+    const auto processOnIntersect =
+        [&result, &closest_dist2, &a, &b, &coll](const Point& p_start, const Point& p_end)
+    {
+        if
+        (
+            LinearAlg2D::lineLineIntersection(a, b, p_start, p_end, coll) &&
+            LinearAlg2D::pointIsProjectedBeyondLine(coll, p_start, p_end) == 0 &&
+            LinearAlg2D::pointIsProjectedBeyondLine(coll, a, b) == 0
+        )
+        {
+            const coord_t dist2 = vSize2(b - coll);
+            if (dist2 < closest_dist2)
+            {
+                closest_dist2 = dist2;
+                result = coll;
+            }
+        }
+    };
+
+    const auto nearby = outline_locator.getNearby(b, within_max_dist);
+    if (! nearby.empty())
+    {
+        for (const auto& pp_idx : nearby)
+        {
+            processOnIntersect(pp_idx.p(), pp_idx.next().p());
+        }
+        if (closest_dist2 < within_max_dist2)
+        {
+            return true;
+        }
+    }
+
+    for (const auto& poly : current_outlines)
+    {
+        const size_t poly_size = poly.size();
+        for (size_t i_segment_start = 0; i_segment_start < poly_size; ++i_segment_start)
+        {
+            const size_t i_segment_end = (i_segment_start + 1) % poly_size;
+            processOnIntersect(poly[i_segment_start], poly[i_segment_end]);
+        }
+    }
+
+    return closest_dist2 < within_max_dist2;
+}
+
 Point PolygonUtils::getVertexInwardNormal(ConstPolygonRef poly, unsigned int point_idx)
 {
     Point p1 = poly[point_idx];
@@ -709,7 +788,7 @@ void PolygonUtils::walkToNearestSmallestConnection(ClosestPolygonPoint& poly1_re
         return;
     }
 
-    int equilibirum_limit = 100; // hard coded value
+    int equilibirum_limit = MM2INT(0.1); // hard coded value
     for (int loop_counter = 0; loop_counter < equilibirum_limit; loop_counter++)
     {
         unsigned int pos1_before = poly1_result.point_idx;
@@ -986,8 +1065,7 @@ std::optional<ClosestPolygonPoint> PolygonUtils::findClose(
     }
     else
     {
-        bool bs_arg = true; // doesn't mean anything. Just to make clear we call the variable arguments of the constructor.
-        return std::optional<ClosestPolygonPoint>(bs_arg, best, best_point_poly_idx.point_idx, polygons[best_point_poly_idx.poly_idx], best_point_poly_idx.poly_idx);
+        return std::optional<ClosestPolygonPoint>(std::in_place, best, best_point_poly_idx.point_idx, polygons[best_point_poly_idx.poly_idx], best_point_poly_idx.poly_idx);
     }
 }
 
@@ -1058,7 +1136,7 @@ bool PolygonUtils::getNextPointWithDistance(Point from, int64_t dist, ConstPolyg
             if (shorterThen(pn, 100)) // when precision is limited
             {
                 Point middle = (next_poly_point + prev_poly_point) / 2;
-                int64_t dist_to_middle = vSize(from - middle);
+                coord_t dist_to_middle = vSize(from - middle);
                 if (dist_to_middle - dist < 100 && dist_to_middle - dist > -100)
                 {
                     result.location = middle;
@@ -1102,6 +1180,27 @@ bool PolygonUtils::getNextPointWithDistance(Point from, int64_t dist, ConstPolyg
     return false;
 }
 
+ClosestPolygonPoint PolygonUtils::walk(const ClosestPolygonPoint& from, coord_t distance)
+{
+    ConstPolygonRef poly = *from.poly;
+    Point last_vertex = from.p();
+    Point next_vertex;
+    size_t last_point_idx = from.point_idx;
+    for (size_t point_idx = from.point_idx + 1; ; point_idx++)
+    {
+        if (point_idx == poly.size())
+        {
+            point_idx = 0;
+        }
+        next_vertex = poly[point_idx];
+        distance -= vSize(last_vertex - next_vertex);
+        if (distance <= 0) break;
+        last_vertex = next_vertex;
+        last_point_idx = point_idx;
+    }
+    Point result = next_vertex + normal(last_vertex - next_vertex, -distance);
+    return ClosestPolygonPoint(result, last_point_idx, poly, from.poly_idx);
+}
 
 std::optional<ClosestPolygonPoint> PolygonUtils::getNextParallelIntersection(const ClosestPolygonPoint& start, const Point& line_to, const coord_t dist, const bool forward)
 {
