@@ -1,16 +1,17 @@
-//Copyright (c) 2019 Ultimaker B.V.
+//Copyright (c) 2021 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
 #ifndef UTILS_POLYGON_UTILS_H
 #define UTILS_POLYGON_UTILS_H
 
-#include <vector>
 #include <functional> // function
+#include <limits>
+#include <optional>
+#include <vector>
 
 #include "polygon.h"
 #include "SparsePointGridInclusive.h"
 #include "SparseLineGrid.h"
-#include "optional.h"
 #include "PolygonsPointIndex.h"
 
 namespace cura 
@@ -37,7 +38,7 @@ struct ClosestPolygonPoint
     {
         return point_idx != NO_INDEX;
     }
-    bool operator==(const ClosestPolygonPoint& rhs)
+    bool operator==(const ClosestPolygonPoint& rhs) const
     {
         // no need to compare on poy_idx
         // it's sometimes unused while poly is always initialized
@@ -135,6 +136,16 @@ public:
      */
     static void spreadDots(PolygonsPointIndex start, PolygonsPointIndex end, unsigned int n_dots, std::vector<ClosestPolygonPoint>& result);
 
+    /*!
+     * Generate a grid of dots inside of the area of the \p polygons.
+     */
+    static std::vector<Point> spreadDotsArea(const Polygons& polygons, coord_t grid_size);
+
+    /*!
+     * Whether a polygon intersects with a line-segment. If true, the closest collision point to 'b' is stored in the result.
+     */
+    static bool lineSegmentPolygonsIntersection(const Point& a, const Point& b, const Polygons& current_outlines, const LocToLineGrid& outline_locator, Point& result, const coord_t within_max_dist);
+
     /*!
      * Get the normal of a boundary point, pointing outward.
      * Only the direction is set.
@@ -480,6 +491,10 @@ public:
     */
     static bool getNextPointWithDistance(Point from, int64_t dist, ConstPolygonRef poly, int start_idx, int poly_start_idx, GivenDistPoint& result);
 
+    /*!
+     * Walk a given \p distance along the polygon from a given point \p from on the polygon
+     */
+    static ClosestPolygonPoint walk(const ClosestPolygonPoint& from, coord_t distance);
 
     /*!
      * Get the point on a polygon which intersects a line parallel to a line going through the starting point and through another point.
